package ca.canadapost.cpcdp.pickup;

import java.nio.file.FileSystems;
import java.nio.file.Path;
import java.util.Map;

import javax.xml.datatype.DatatypeFactory;
import javax.xml.namespace.QName;
import javax.xml.ws.BindingProvider;
import javax.xml.ws.Service;
import javax.xml.ws.soap.SOAPFaultException;

import org.apache.cxf.endpoint.Client;
import org.apache.cxf.frontend.ClientProxy;

import ca.canadapost.cpcdp.pickup.stub.pickuprequest.GetPickupPriceRequest;
import ca.canadapost.cpcdp.pickup.stub.pickuprequest.GetPickupPriceResponse;
import ca.canadapost.cpcdp.pickup.stub.pickuprequest.PickupDetailsType;
import ca.canadapost.cpcdp.pickup.stub.pickuprequest.PickupRequestPortType;
import ca.canadapost.cpcdp.pickup.stub.pickuprequest.PickupRequestPriceNoCCType;
import ca.canadapost.cpcdp.pickup.util.ClientUtil;

/**
 * Sample code for the GetPickupPriceRequest Canada Post service.
 * <p>
 * The GetPickupPriceRequest service returns the price of an on-demand pickup.
 * <p>
 * This sample is configured to access the Developer Program sandbox environment.
 * Use your development key username and password for the web service credentials.
 */
public class GetPickupPriceRequestClient {
	// Main class for local testing only
	public static void main(String[] args) {
		try {
			// Your username, password and customer number are imported from the following file: CPCWS_SOAP_Pickup_Java_Samples/user.properties
            Map<String, String> userProperties = ClientUtil.getUserProperties("user.properties");
            String username = userProperties.get("username");
            String password = userProperties.get("password");
            String customer = userProperties.get("mailedBy");
            
            QName serviceQName = new QName("http://www.canadapost.ca/ws/soap/pickuprequest", "PickupRequestService");
            Path wsdlPath = FileSystems.getDefault().getPath("SOAP/wsdl", "pickuprequest.wsdl");
            Service service = Service.create(wsdlPath.toUri().toURL(), serviceQName);
            PickupRequestPortType port = service.getPort(PickupRequestPortType.class);

            //set endpoint
            Map ctx = ((BindingProvider) port).getRequestContext();
            ctx.put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, "https://ct.soa-gw.canadapost.ca/enab/soap/pickuprequest");

            //wss4j interceptor -  add UsernameToken to soap header
            Client client = ClientProxy.getClient(port);
            client.getOutInterceptors().add(ClientUtil.getUsernameTokenInterceptor(username, password));
            
            //log request/response
            //client.getInInterceptors().add(new LoggingInInterceptor());
            //client.getOutInterceptors().add(new LoggingOutInterceptor());

            //build request
            PickupDetailsType pickupDetailsType = new PickupDetailsType();
            DatatypeFactory df = DatatypeFactory.newInstance();
            pickupDetailsType.setDate(df.newXMLGregorianCalendar("2019-02-22"));
            
            GetPickupPriceRequest request = new GetPickupPriceRequest();
            request.setLocale("FR");
            request.setCustomerNumber(customer);
            request.setPickupDetails(pickupDetailsType);

            //execute request
            GetPickupPriceResponse response = port.getPickupPrice(request);

            //retrieve values from response
            if(response.getMessages() == null){
            	PickupRequestPriceNoCCType price = response.getPickupRequestPrice();
                System.out.println("Pre-tax Amount:\t" + price.getPreTaxAmount());
        		System.out.println("GST Amount:\t" + price.getGstAmount());
        		System.out.println("PST Amount:\t" + price.getPstAmount());
        		System.out.println("HST Amount:\t" + price.getHstAmount());
        		System.out.println("Due Amount:\t" + price.getDueAmount());
            } else {
                /*for(Messages.Message m : response.getMessages().getMessage()){
                    System.out.println("Error Code: " + m.getCode());
                    System.out.println("Error Message: " + m.getDescription());
                }*/
            }
        }
        catch (SOAPFaultException e) {
            System.out.println("Fault Code: " + e.getFault().getFaultCode());
            System.out.println("Fault Reason: " + e.getFault().getFaultString());
        } catch (Exception e) {
            e.printStackTrace();
        }
	}
}
